@@ -2,7 +2,7 @@
  * ac3_imdct.h : AC3 IMDCT types
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: ac3_imdct.h,v 1.5 2001/07/08 23:15:11 reno Exp $
+ * $Id: ac3_imdct.h,v 1.6 2001/10/30 19:34:53 reno Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Renaud Dartus <reno@videolan.org>
@@ -31,30 +31,27 @@ typedef struct complex_s {
 
 typedef struct imdct_s
 {
-    complex_t buf[N/4] __attribute__ ((aligned(16)));
+    complex_t * buf;
 
     /* Delay buffer for time domain interleaving */
-    float delay[6][256] __attribute__ ((aligned(16)));
-    float delay1[6][256] __attribute__ ((aligned(16)));
+    float * delay;
+    float * delay1;
 
     /* Twiddle factors for IMDCT */
-    float xcos1[N/4] __attribute__ ((aligned(16)));
-    float xsin1[N/4] __attribute__ ((aligned(16)));
-    float xcos2[N/8] __attribute__ ((aligned(16)));
-    float xsin2[N/8] __attribute__ ((aligned(16)));
-    float xcos_sin_sse[128 * 4] __attribute__ ((aligned(16)));
+    float * xcos1;
+    float * xsin1;
+    float * xcos2;
+    float * xsin2;
+    float * xcos_sin_sse;
    
     /* Twiddle factor LUT */
-    complex_t w_1[1] __attribute__ ((aligned(16)));
-    float used_for_alignement1;
-    float used_for_alignement2;
-    complex_t w_2[2] __attribute__ ((aligned(16)));
-    complex_t w_4[4] __attribute__ ((aligned(16)));
-    complex_t w_8[8] __attribute__ ((aligned(16)));
-    complex_t w_16[16] __attribute__ ((aligned(16)));
-    complex_t w_32[32] __attribute__ ((aligned(16)));
-    complex_t w_64[64] __attribute__ ((aligned(16)));
-    complex_t *w[7] __attribute__ ((aligned(16)));
+    complex_t * w_2;
+    complex_t * w_4;
+    complex_t * w_8;
+    complex_t * w_16;
+    complex_t * w_32;
+    complex_t * w_64;
+    complex_t * w_1;
     
     /* Module used and shortcuts */
     struct module_s * p_module;
