@@ -5,7 +5,7 @@
  * contains the basic udf handling functions
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: dvd_udf.c,v 1.12 2001/07/27 01:05:17 sam Exp $
+ * $Id: dvd_udf.c,v 1.15 2001/11/07 17:37:16 stef Exp $
  *
  * Author: Stphane Borel <stef@via.ecp.fr>
  *
@@ -50,7 +50,11 @@
 #   include <strings.h>
 #endif
 
-#include <videolan/dvdcss.h>
+#ifdef GOD_DAMN_DMCA
+#   include "dummy_dvdcss.h"
+#else
+#   include <videolan/dvdcss.h>
+#endif
 
 #include "config.h"
 #include "common.h"
@@ -105,7 +109,7 @@ typedef struct ad_s
 static int UDFReadLB( dvdcss_handle dvdhandle, off_t i_lba,
                       size_t i_block_count, u8 *pi_data )
 {
-    if( dvdcss_seek( dvdhandle, i_lba ) < 0 )
+    if( dvdcss_seek( dvdhandle, i_lba, DVDCSS_NOFLAGS ) < 0 )
     {
         intf_ErrMsg( "UDF: Postion not found" );
         return 0;
@@ -644,13 +648,13 @@ static int UDFFindPartition( int i_part_nb, struct partition_s *p_partition )
 
 
 /*****************************************************************************
- * UDFFindFile: looks for a file on the UDF disc/imagefile
+ * DVDUDFFindFile: looks for a file on the UDF disc/imagefile
  *****************************************************************************
  * Path has to be the absolute pathname on the UDF filesystem,
  * starting with '/'.
  * returns absolute LB number, or 0 on error
  *****************************************************************************/
-u32 UDFFindFile( dvdcss_handle dvdhandle, char * psz_path )
+u32 DVDUDFFindFile( dvdcss_handle dvdhandle, char * psz_path )
 {
     struct partition_s  partition;
     struct ad_s         root_icb;
