@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input.c,v 1.142 2001/10/10 14:25:15 sam Exp $
+ * $Id: input.c,v 1.154 2001/11/12 22:42:56 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -52,9 +52,9 @@
 #   include <netdb.h>                                         /* hostent ... */
 #   include <sys/socket.h>
 #   include <netinet/in.h>
-#   include <arpa/inet.h>
-#   include <sys/types.h>
-#   include <sys/socket.h>
+#   ifdef HAVE_ARPA_INET_H
+#       include <arpa/inet.h>                    /* inet_ntoa(), inet_aton() */
+#   endif
 #endif
 
 #ifdef HAVE_SYS_TIMES_H
@@ -86,10 +86,12 @@
 static void RunThread       ( input_thread_t *p_input );
 static  int InitThread      ( input_thread_t *p_input );
 static void ErrorThread     ( input_thread_t *p_input );
+static void CloseThread     ( input_thread_t *p_input );
 static void DestroyThread   ( input_thread_t *p_input );
 static void EndThread       ( input_thread_t *p_input );
 
 static void FileOpen        ( input_thread_t *p_input );
+static void StdOpen         ( input_thread_t *p_input );
 static void FileClose       ( input_thread_t *p_input );
 #if !defined( SYS_BEOS ) && !defined( SYS_NTO )
 static void NetworkOpen     ( input_thread_t *p_input );
@@ -413,17 +415,18 @@ static int InitThread( input_thread_t * p_input )
     p_input->pf_delete_pes    = f.pf_delete_pes;
     p_input->pf_rewind        = f.pf_rewind;
     p_input->pf_seek          = f.pf_seek;
-#undef f
 
 #if !defined( SYS_BEOS ) && !defined( SYS_NTO )
     /* FIXME : this is waaaay too kludgy */
-    if( (strlen( p_input->p_source ) > 3) && !strncasecmp( p_input->p_source, "ts:", 3 ) )
+    if( ( strlen( p_input->p_source ) > 3)
+          && !strncasecmp( p_input->p_source, "ts:", 3 ) )
     {
         /* Network stream */
         NetworkOpen( p_input );
         p_input->stream.i_method = INPUT_METHOD_NETWORK;
     }
-    else if( ( strlen( p_input->p_source ) > 5 ) && !strncasecmp( p_input->p_source, "http:", 5 ) )
+    else if( ( strlen( p_input->p_source ) > 5 )
+               && !strncasecmp( p_input->p_source, "http:", 5 ) )
     {
         /* HTTP stream */
         HTTPOpen( p_input );
@@ -431,16 +434,24 @@ static int InitThread( input_thread_t * p_input )
     }
     else 
 #endif
-        if( ( strlen( p_input->p_source ) > 4 ) && !strncasecmp( p_input->p_source, "dvd:", 4 ) )
+        if( ( strlen( p_input->p_source ) > 4 )
+              && !strncasecmp( p_input->p_source, "dvd:", 4 ) )
     {
         /* DVD - this is THE kludge */
-        p_input->p_input_module->p_functions->input.functions.input.pf_open( p_input );
+        f.pf_open( p_input );
         p_input->stream.i_method = INPUT_METHOD_DVD;
     }
-    else if( ( strlen( p_input->p_source ) > 4 ) && !strncasecmp( p_input->p_source, "vlc:", 4 ) )
+    else if( ( strlen( p_input->p_source ) > 4 )
+               && !strncasecmp( p_input->p_source, "vlc:", 4 ) )
     {
         /* Dummy input - very kludgy */
-        p_input->p_input_module->p_functions->input.functions.input.pf_open( p_input );
+        f.pf_open( p_input );
+    }
+    else if( ( strlen( p_input->p_source ) == 1 )
+               && *p_input->p_source == '-' )
+    {
+        /* Stdin */
+        StdOpen( p_input );
     }
     else
     {
@@ -448,6 +459,7 @@ static int InitThread( input_thread_t * p_input )
         FileOpen( p_input );
         p_input->stream.i_method = INPUT_METHOD_FILE;
     }
+#undef f
 
     if( p_input->b_error )
     {
@@ -461,7 +473,7 @@ static int InitThread( input_thread_t * p_input )
     if( p_input->b_error )
     {
         /* We barfed -- exit nicely */
-        p_input->pf_close( p_input );
+        CloseThread( p_input );
         module_Unneed( p_input->p_input_module );
         return( -1 );
     }
@@ -519,34 +531,49 @@ static void EndThread( input_thread_t * p_input )
     /* Free demultiplexer's data */
     p_input->pf_end( p_input );
 
+    /* Close the input method */
+    CloseThread( p_input );
+
+    /* Release modules */
+    module_Unneed( p_input->p_input_module );
+}
+
+/*****************************************************************************
+ * CloseThread: close the target
+ *****************************************************************************/
+static void CloseThread( input_thread_t * p_input )
+{
+#define f p_input->p_input_module->p_functions->input.functions.input
+
 #if !defined( SYS_BEOS ) && !defined( SYS_NTO )
     /* Close stream */
-    if( (strlen( p_input->p_source ) > 3) && !strncasecmp( p_input->p_source, "ts:", 3 ) )
+    if( ( strlen( p_input->p_source ) > 3)
+          && !strncasecmp( p_input->p_source, "ts:", 3 ) )
     {
         NetworkClose( p_input );
     }
-    else if( ( strlen( p_input->p_source ) > 5 ) && !strncasecmp( p_input->p_source, "http:", 5 ) )
+    else if( ( strlen( p_input->p_source ) > 5 )
+               && !strncasecmp( p_input->p_source, "http:", 5 ) )
     {
         NetworkClose( p_input );
     }
     else 
 #endif
-    if( ( strlen( p_input->p_source ) > 4 ) && !strncasecmp( p_input->p_source, "dvd:", 4 ) )
+    if( ( strlen( p_input->p_source ) > 4 )
+          && !strncasecmp( p_input->p_source, "dvd:", 4 ) )
     {
-        p_input->p_input_module->p_functions->input.functions.input.pf_close( p_input );
+        f.pf_close( p_input );
     }
-    else if( ( strlen( p_input->p_source ) > 4 ) && !strncasecmp( p_input->p_source, "vlc:", 4 ) )
+    else if( ( strlen( p_input->p_source ) > 4 )
+               && !strncasecmp( p_input->p_source, "vlc:", 4 ) )
     {
-        p_input->p_input_module->p_functions->input.functions.input.pf_close( p_input );
+        f.pf_close( p_input );
     }
     else
     {
         FileClose( p_input );
     }
-
-    /* Release modules */
-    module_Unneed( p_input->p_input_module );
-
+#undef f
 }
 
 /*****************************************************************************
@@ -570,6 +597,25 @@ static void DestroyThread( input_thread_t * p_input )
     *pi_status = THREAD_OVER;
 }
 
+/*****************************************************************************
+ * StdOpen : open standard input
+ *****************************************************************************/
+static void StdOpen( input_thread_t * p_input )
+{
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
+    /* Suppose we can control the pace - this won't work in some cases ! */
+    p_input->stream.b_pace_control = 1;
+
+    p_input->stream.b_seekable = 0;
+    p_input->stream.p_selected_area->i_size = 0;
+    p_input->stream.p_selected_area->i_tell = 0;
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
+    intf_WarnMsg( 2, "input: opening stdin" );
+    p_input->i_handle = 0;
+}
+
 /*****************************************************************************
  * FileOpen : open a file descriptor
  *****************************************************************************/
@@ -591,6 +637,13 @@ static void FileOpen( input_thread_t * p_input )
             psz_name += 4;
             i_stat = stat( psz_name, &stat_info );
         }
+        else if( ( i_size > 4 )
+                 && !strncasecmp( psz_name, "vcd:", 4 ) )
+        {
+            /* get rid of the 'vcd:' stuff and try again */
+            psz_name += 4;
+            i_stat = stat( psz_name, &stat_info );
+        }
         else if( ( i_size > 5 )
                  && !strncasecmp( psz_name, "file:", 5 ) )
         {
@@ -677,22 +730,6 @@ static void NetworkOpen( input_thread_t * p_input )
     struct sockaddr_in  sock;
     unsigned int        i_mc_group;
 
-#ifdef WIN32
-    WSADATA Data;
-    int i_err;
-#endif
-    
-#ifdef WIN32
-    /* WinSock Library Init. */
-    i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
-
-    if( i_err )
-    {
-        intf_ErrMsg( "input: can't initiate WinSocks, error %i", i_err );
-        return ;
-    }
-#endif
-    
     /* Get the remote server */
     if( p_input->p_source != NULL )
     {
@@ -716,18 +753,24 @@ static void NetworkOpen( input_thread_t * p_input )
             char *psz_port = psz_server;
 
             /* Skip the hostname part */
-            while( *psz_port && *psz_port != ':' )
+            while( *psz_port && *psz_port != ':' && *psz_port != '/' )
             {
                 psz_port++;
             }
 
-            /* Found a port name */
+            /* Found a port name or a broadcast addres */
             if( *psz_port )
             {
-                /* Replace ':' with '\0' */
-                *psz_port = '\0';
-                psz_port++;
+                /* That's a port name */
+                if( *psz_port == ':' )
+                {
+                    *psz_port = '\0';
+                    psz_port++;
+                    i_port = atoi( psz_port );
+                }
 
+                /* Search for '/' just after the port in case
+                 * we also have a broadcast address */
                 psz_broadcast = psz_port;
                 while( *psz_broadcast && *psz_broadcast != '/' )
                 {
@@ -747,12 +790,6 @@ static void NetworkOpen( input_thread_t * p_input )
                 {
                     psz_broadcast = NULL;
                 }
-
-                /* port before broadcast address */
-                if( *psz_port != '\0' )
-                {
-                    i_port = atoi( psz_port );
-                }
             }
         }
         else
@@ -760,6 +797,11 @@ static void NetworkOpen( input_thread_t * p_input )
             psz_server = NULL;
         }
     }
+    else
+    {
+        /* This is required or NetworkClose will never be called */
+        p_input->p_source = "ts: network input";
+    }
 
     /* Check that we got a valid server */
     if( psz_server == NULL )
@@ -785,7 +827,7 @@ static void NetworkOpen( input_thread_t * p_input )
         }
         else
         {
-           psz_broadcast = NULL; 
+            psz_broadcast = NULL; 
         }
     }
 
@@ -861,13 +903,11 @@ static void NetworkOpen( input_thread_t * p_input )
     i_mc_group = sock.sin_addr.s_addr;
 
 #if defined( WIN32 )
-    if ( psz_broadcast != NULL )
-    {
-        sock.sin_addr.s_addr = INADDR_ANY;
-    }
+    sock.sin_addr.s_addr = INADDR_ANY;
+    
 #define IN_MULTICAST(a)         IN_CLASSD(a)
 #endif
-    
+
     /* Bind it */
     if( bind( p_input->i_handle, (struct sockaddr *)&sock, 
               sizeof( sock ) ) < 0 )
@@ -879,6 +919,8 @@ static void NetworkOpen( input_thread_t * p_input )
     }
 
     /* Join the multicast group if the socket is a multicast address */
+
+#ifndef WIN32    
     if( IN_MULTICAST( ntohl(i_mc_group) ) )
     {
         struct ip_mreq imr;
@@ -895,7 +937,7 @@ static void NetworkOpen( input_thread_t * p_input )
             return;
         }
     }
-
+    
     /* Build socket for remote connection */
     if ( network_BuildRemoteAddr( &sock, psz_server ) == -1 )
     {
@@ -919,6 +961,7 @@ static void NetworkOpen( input_thread_t * p_input )
             return;
         }
     }
+#endif
 
     p_input->stream.b_pace_control = 0;
     p_input->stream.b_seekable = 0;
@@ -933,11 +976,9 @@ static void NetworkOpen( input_thread_t * p_input )
  *****************************************************************************/
 static void NetworkClose( input_thread_t * p_input )
 {
-    close( p_input->i_handle );
+    intf_WarnMsg( 2, "input: closing network target `%s'", p_input->p_source );
 
-#ifdef WIN32 
-    WSACleanup();
-#endif
+    close( p_input->i_handle );
 }
 
 /*****************************************************************************
@@ -953,22 +994,6 @@ static void HTTPOpen( input_thread_t * p_input )
     struct sockaddr_in  sock;
     char                psz_buffer[256];
 
-#ifdef WIN32
-    WSADATA Data;
-    int i_err;
-#endif
-    
-#ifdef WIN32
-    /* WinSock Library Init. */
-    i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
-
-    if( i_err )
-    {
-        intf_ErrMsg( "input: can't initiate WinSocks, error %i", i_err );
-        return ;
-    }
-#endif
-    
     /* Get the remote server */
     if( p_input->p_source != NULL )
     {
