@@ -2,7 +2,7 @@
  * mpeg_system.c: TS, PS and PES management
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: mpeg_system.c,v 1.56 2001/07/18 15:21:51 massiot Exp $
+ * $Id: mpeg_system.c,v 1.64 2001/11/12 10:33:42 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Lespinasse <walken@via.ecp.fr>
@@ -46,7 +46,7 @@
 #include "input_ext-dec.h"
 #include "input_ext-plugins.h"
 
-#include "main.h"                           /* AC3/MPEG channel, SPU channel */
+#include "main.h"                   /* AC3/MPEG channel, SPU channel, b_stat */
 
 /*****************************************************************************
  * Local prototypes
@@ -149,7 +149,9 @@ void input_ParsePES( input_thread_t * p_input, es_descriptor_t * p_es )
     /* Get the PES size if defined */
     p_es->i_pes_real_size = U16_AT(p_header + 4);
     if( p_es->i_pes_real_size )
+    {
         p_es->i_pes_real_size += 6;
+    }
 
     /* First read the 6 header bytes common to all PES packets:
      * use them to test the PES validity */
@@ -641,10 +643,11 @@ static void DecodePSM( input_thread_t * p_input, data_packet_t * p_data )
                      p_input->stream.pp_programs[0]->pp_es[i_new_es_number] );
     }
 
-#ifdef STATS
-    intf_Msg( "input info: The stream map after the PSM is now :" );
-    input_DumpStream( p_input );
-#endif
+    if( p_main->b_stats )
+    {
+        intf_StatMsg( "input info: The stream map after the PSM is now :" );
+        input_DumpStream( p_input );
+    }
 
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 }
@@ -903,9 +906,7 @@ void input_DemuxPS( input_thread_t * p_input, data_packet_t * p_data )
              && (!p_es->b_audio || !p_input->stream.control.b_mute) )
         {
             vlc_mutex_unlock( &p_input->stream.control.control_lock );
-#ifdef STATS
             p_es->c_packets++;
-#endif
             input_GatherPES( p_input, p_data, p_es, 1, 0 );
         }
         else
@@ -919,9 +920,7 @@ void input_DemuxPS( input_thread_t * p_input, data_packet_t * p_data )
     if( b_trash )
     {
         p_input->pf_delete_packet( p_input->p_method_data, p_data );
-#ifdef STATS
-        p_input->c_packets_trashed++;
-#endif
+        p_input->stream.c_packets_trashed++;
     }
 }
 
@@ -947,9 +946,9 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
     es_ts_data_t *      p_es_demux = NULL;
     pgrm_ts_data_t *    p_pgrm_demux = NULL;
 
-    #define p (p_data->p_buffer)
+#define p (p_data->p_buffer)
     /* Extract flags values from TS common header. */
-    i_pid = U16_AT(&p[1]) & 0x1fff;
+    i_pid = ((p[1] & 0x1F) << 8) | p[2];
     b_unit_start = (p[1] & 0x40);
     b_adaptation = (p[3] & 0x20);
     b_payload = (p[3] & 0x10);
@@ -964,9 +963,13 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
         p_es_demux = (es_ts_data_t *)p_es->p_demux_data;
         
         if( p_es_demux->b_psi )
+        {
             b_psi = 1;
+        }
         else
+        {
             p_pgrm_demux = (pgrm_ts_data_t *)p_es->p_pgrm->p_demux_data; 
+        }
     }
 
     vlc_mutex_lock( &p_input->stream.control.control_lock );
@@ -975,22 +978,23 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
         /* Not selected. Just read the adaptation field for a PCR. */
         b_trash = 1;
     }
-    else if( p_es->p_decoder_fifo == NULL  && !b_psi )
-      b_trash =1; 
+    else if( p_es->p_decoder_fifo == NULL && !b_psi )
+    {
+        b_trash = 1; 
+    }
 
     vlc_mutex_unlock( &p_input->stream.control.control_lock );
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
 
     /* Don't change the order of the tests : if b_psi then p_pgrm_demux 
-     * may still be null. Who said it was ugly ? */
+     * may still be null. Who said it was ugly ?
+     * I have written worse. --Meuuh */
     if( ( p_es != NULL ) && 
         ((p_es->p_decoder_fifo != NULL) || b_psi 
                                    || (p_pgrm_demux->i_pcr_pid == i_pid) ) )
     {
-#ifdef STATS
         p_es->c_packets++;
-#endif
 
         /* Extract adaptation field information if any */
 
@@ -1017,12 +1021,10 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
                 if( b_payload ? (p[4] > 182) : (p[4] != 183) )
                 {
                     intf_WarnMsg( 2,
-                        "invalid TS adaptation field (%p)",
+                        "Invalid TS adaptation field (%p)",
                         p_data );
                     p_data->b_discard_payload = 1;
-#ifdef STATS
                     p_es->c_invalid_packets++;
-#endif
                 }
     
                 /* Now we are sure that the byte containing flags is present:
@@ -1044,7 +1046,7 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
                         p_es->p_pgrm->i_synchro_state = SYNCHRO_REINIT;
     
                         /* There also may be a continuity_counter
-                         * discontinuity: resynchronise our counter with
+                         * discontinuity: resynchronize our counter with
                          * the one of the stream. */
                         p_es_demux->i_continuity_counter = (p[3] & 0x0f) - 1;
                     }
@@ -1052,7 +1054,8 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
                     /* If this is a PCR_PID, and this TS packet contains a
                      * PCR, we pass it along to the PCR decoder. */
 
-                    if( !b_psi && (p_pgrm_demux->i_pcr_pid == i_pid) && (p[5] & 0x10) )
+                    if( !b_psi && (p_pgrm_demux->i_pcr_pid == i_pid)
+                        && (p[5] & 0x10) )
                     {
                         /* There should be a PCR field in the packet, check
                          * if the adaptation field is long enough to carry
@@ -1061,9 +1064,11 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
                         {
                             /* Read the PCR. */
                             mtime_t     pcr_time;
-                            pcr_time =
-                                    ( (mtime_t)U32_AT((u32*)&p[6]) << 1 )
-                                      | ( p[10] >> 7 );
+                            pcr_time = ( (mtime_t)p[6] << 25 ) |
+                                       ( (mtime_t)p[7] << 17 ) |
+                                       ( (mtime_t)p[8] << 9 ) |
+                                       ( (mtime_t)p[9] << 1 ) |
+                                       ( (mtime_t)p[10] >> 7 );
                             /* Call the pace control. */
                             input_ClockManageRef( p_input, p_es->p_pgrm,
                                                   pcr_time );
@@ -1093,7 +1098,6 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
             }
             else if( i_dummy <= 0 )
             {
-                /* FIXME: this can never happen, can it ? --Meuuh */
                 /* Duplicate packet: mark it as being to be trashed. */
                 intf_WarnMsg( 3, "Duplicate packet received by TS demux" );
                 b_trash = 1;
@@ -1104,8 +1108,8 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
                  * this ES since the continuity counter ranges between 0 and
                  * 0x0F excepts when it has been initialized by the input:
                  * init the counter to the correct value. */
-                intf_DbgMsg( "First packet for PID %d received by TS demux",
-                             p_es->i_id );
+                intf_WarnMsg( 3, "First packet for PID %d received by TS demux",
+                              p_es->i_id );
                 p_es_demux->i_continuity_counter = (p[3] & 0x0f);
             }
             else
@@ -1128,9 +1132,7 @@ void input_DemuxTS( input_thread_t * p_input, data_packet_t * p_data )
     if( b_trash )
     {
         p_input->pf_delete_packet( p_input->p_method_data, p_data );
-#ifdef STATS
-        p_input->c_packets_trashed++;
-#endif
+        p_input->stream.c_packets_trashed++;
     }
     else
     {
@@ -1181,7 +1183,9 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
             p+=(u8)p[0];
         }
         else
+        {
             p++;
+        }
 
         /* This is the begining of a new section */
 
@@ -1192,7 +1196,7 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
         }
         else 
         {
-            p_psi->i_section_length = U16_AT(p+1) & 0x0fff;
+            p_psi->i_section_length = ((p[1] & 0xF) << 8) | p[2];
             p_psi->b_section_complete = 0;
             p_psi->i_read_in_section = 0;
             p_psi->i_section_number = (u8)p[6];
@@ -1217,7 +1221,8 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
                     
                     if( p_psi->i_version_number != (( p[5] >> 1 ) & 0x1f) )
                     {
-                        intf_WarnMsg( 2,"PSI version differs inside same PAT" );
+                        intf_WarnMsg( 2,
+                                      "PSI version differs inside same PAT" );
                         p_psi->b_trash = 1;
                     }
                     if( p_psi->i_section_number + 1 != (u8)p[6] )
@@ -1256,6 +1261,18 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
             {
                 /* This was the last section of PSI */
                 p_psi->b_is_complete = 1;
+
+                switch( p_demux_data->i_psi_type)
+                {
+                case PSI_IS_PAT:
+                    input_DecodePAT( p_input, p_es );
+                    break;
+                case PSI_IS_PMT:
+                    input_DecodePMT( p_input, p_es );
+                    break;
+                default:
+                    intf_WarnMsg(2, "Received unknown PSI in DemuxPSI");
+                }
             }
         }
         else
@@ -1267,22 +1284,10 @@ void input_DemuxPSI( input_thread_t * p_input, data_packet_t * p_data,
         }
     }
 
-    if ( p_psi->b_is_complete )
-    {
-        switch( p_demux_data->i_psi_type)
-        {
-            case PSI_IS_PAT:
-                input_DecodePAT( p_input, p_es );
-                break;
-            case PSI_IS_PMT:
-                input_DecodePMT( p_input, p_es );
-                break;
-            default:
-                intf_WarnMsg(2, "Received unknown PSI in demuxPSI");
-        }
-    }
 #undef p_psi    
 #undef p
+   
+    p_input->pf_delete_packet( p_input->p_method_data, p_data );
     
     return ;
 }
@@ -1326,13 +1331,17 @@ static void input_DecodePAT( input_thread_t * p_input, es_descriptor_t * p_es )
         
         do
         {
-            i_section_length = U16_AT(p_current_data+1) & 0x0fff;
+            i_section_length = ((p_current_data[1] & 0xF) << 8) |
+                                 p_current_data[2];
             i_current_section = (u8)p_current_data[6];
     
             for( i_loop = 0; i_loop < (i_section_length-9)/4 ; i_loop++ )
             {
-                i_program_id = U16_AT(p_current_data + i_loop*4 + 8);
-                i_pmt_pid = U16_AT( p_current_data + i_loop*4 + 10) & 0x1fff;
+                i_program_id = ( *(p_current_data + i_loop * 4 + 8) << 8 ) |
+                                 *(p_current_data + i_loop * 4 + 9);
+                i_pmt_pid = ( (*( p_current_data + i_loop * 4 + 10) & 0x1F)
+                                    << 8 ) |
+                               *( p_current_data + i_loop * 4 + 11);
     
                 /* If program = 0, we're having info about NIT not PMT */
                 if( i_program_id )
@@ -1401,12 +1410,13 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
         
         p_current_section = p_psi->buffer;
         p_current_data = p_psi->buffer;
-        
-        p_pgrm_data->i_pcr_pid = U16_AT(p_current_section + 8) & 0x1fff;
-        
+
+        p_pgrm_data->i_pcr_pid = ( (*(p_current_section + 8) & 0x1F) << 8 ) |
+                                    *(p_current_section + 9);
+
         i_audio_es = 0;
         i_spu_es = 0;
-        
+
         /* Lock stream information */
         vlc_mutex_lock( &p_input->stream.stream_lock );
 
@@ -1454,21 +1464,24 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
         /* Then add what we received in this PMT */
         do
         {
-            
-            i_section_length = U16_AT(p_current_data+1) & 0x0fff;
+            i_section_length = ( (*(p_current_data + 1) & 0xF) << 8 ) |
+                                  *(p_current_data + 2);
             i_current_section = (u8)p_current_data[6];
-            i_prog_info_length = U16_AT(p_current_data+10) & 0x0fff;
+            i_prog_info_length = ( (*(p_current_data + 10) & 0xF) << 8 ) |
+                                    *(p_current_data + 11);
 
             /* For the moment we ignore program descriptors */
-            p_current_data += 12+i_prog_info_length;
+            p_current_data += 12 + i_prog_info_length;
     
             /* The end of the section, before the CRC is at 
              * p_current_section + i_section_length -1 */
             while( p_current_data < p_current_section + i_section_length -1 )
             {
                 i_stream_type = (int)p_current_data[0];
-                i_pid = U16_AT( p_current_data + 1 ) & 0x1fff;
-                i_es_info_length = U16_AT( p_current_data + 3 ) & 0x0fff;
+                i_pid = ( (*(p_current_data + 1) & 0x1F) << 8 ) |
+                           *(p_current_data + 2);
+                i_es_info_length = ( (*(p_current_data + 3) & 0xF) << 8 ) |
+                                      *(p_current_data + 4);
                 
                 /* Add this ES to the program */
                 p_new_es = input_AddES( p_input, p_es->p_pgrm, 
@@ -1488,12 +1501,12 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                         break;
                     case MPEG1_AUDIO_ES:
                     case MPEG2_AUDIO_ES:
-                    case LPCM_AUDIO_ES :
                         p_new_es->i_cat = AUDIO_ES;
                         i_audio_es += 1;
                         if( i_audio_es == i_required_audio_es )
                             input_SelectES( p_input, p_new_es );
                         break;
+                    case LPCM_AUDIO_ES :
                     case AC3_AUDIO_ES :
                         p_new_es->i_stream_id = 0xBD;
                         p_new_es->i_cat = AUDIO_ES;
@@ -1501,8 +1514,9 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
                         if( i_audio_es == i_required_audio_es )
                             input_SelectES( p_input, p_new_es );
                         break;
-                    /* Not sure this one is fully norm-compliant */
+                    /* Not sure this one is fully specification-compliant */
                     case DVD_SPU_ES :
+                        p_new_es->i_stream_id = 0xBD;
                         p_new_es->i_cat = SPU_ES;
                         i_spu_es += 1;
                         if( i_spu_es == i_required_spu_es )
@@ -1517,26 +1531,29 @@ static void input_DecodePMT( input_thread_t * p_input, es_descriptor_t * p_es )
             }
 
             /* Go to the beginning of the next section*/
-            p_current_data += 3+i_section_length;
+            p_current_data += 3 + i_section_length;
            
-            p_current_section+=1;
+            p_current_section++;
             
         } while( i_current_section < p_psi->i_last_section_number );
 
         if( i_required_audio_es > i_audio_es )
-            intf_WarnMsg( 2, "TS input: Non-existing audio es required." );
+        {
+            intf_WarnMsg( 2, "TS input: Non-existing audio ES required." );
+        }
         
         if( i_required_spu_es > i_spu_es )
-            intf_WarnMsg( 2, "TS input: Non-existing subtitles es required." );
+        {
+            intf_WarnMsg( 2, "TS input: Non-existing subtitles ES required." );
+        }
         
         p_pgrm_data->i_pmt_version = p_psi->i_version_number;
 
         /* inform interface that stream has changed */
         p_input->stream.b_changed = 1;
+        /*  Remove lock */
+        vlc_mutex_unlock( &p_input->stream.stream_lock );
     }
     
 #undef p_psi
-
-    /*  Remove lock */
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
 }
