@@ -4,7 +4,7 @@
  * and spawn threads.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: main.c,v 1.117 2001/10/03 15:10:55 sam Exp $
+ * $Id: main.c,v 1.124 2001/11/12 22:42:56 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -92,6 +92,10 @@
 #   include "darwin_specific.h"
 #endif
 
+#ifdef WIN32
+#   include "win32_specific.h"
+#endif
+
 #include "netutils.h"                                 /* network_ChannelJoin */
 
 #include "main.h"
@@ -107,6 +111,7 @@
 #define OPT_STEREO              151
 #define OPT_MONO                152
 #define OPT_SPDIF               153
+#define OPT_VOLUME              154
 
 #define OPT_NOVIDEO             160
 #define OPT_DISPLAY             161
@@ -122,6 +127,7 @@
 #define OPT_SERVER              171
 #define OPT_PORT                172
 #define OPT_BROADCAST           173
+#define OPT_CHANNELSERVER       174
 
 #define OPT_INPUT               180
 #define OPT_MOTION              181
@@ -129,6 +135,8 @@
 #define OPT_YUV                 183
 #define OPT_DOWNMIX             184
 #define OPT_IMDCT               185
+#define OPT_DVDCSS_METHOD       186
+#define OPT_DVDCSS_VERBOSE      187
 
 #define OPT_SYNCHRO             190
 #define OPT_WARNING             191
@@ -175,6 +183,7 @@ static const struct option longopts[] =
     {   "spdif",            0,          0,      OPT_SPDIF },
     {   "downmix",          1,          0,      OPT_DOWNMIX },
     {   "imdct",            1,          0,      OPT_IMDCT },
+    {   "volume",           1,          0,      OPT_VOLUME },
 
     /* Video options */
     {   "novideo",          0,          0,      OPT_NOVIDEO },
@@ -199,6 +208,8 @@ static const struct option longopts[] =
     {   "dvdaudio",         1,          0,      'a' },
     {   "dvdchannel",       1,          0,      'c' },
     {   "dvdsubtitle",      1,          0,      's' },
+    {   "dvdcss-method",    1,          0,      OPT_DVDCSS_METHOD },
+    {   "dvdcss-verbose",   1,          0,      OPT_DVDCSS_VERBOSE },
     
     /* Input options */
     {   "input",            1,          0,      OPT_INPUT },
@@ -206,6 +217,7 @@ static const struct option longopts[] =
     {   "port",             1,          0,      OPT_PORT },
     {   "broadcast",        1,          0,      OPT_BROADCAST },
     {   "channels",         0,          0,      OPT_CHANNELS },
+    {   "channelserver",    1,          0,      OPT_CHANNELSERVER },
 
     /* Synchro options */
     {   "synchro",          1,          0,      OPT_SYNCHRO },
@@ -280,7 +292,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * System specific initialization code
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN ) || defined( WIN32 )
     system_Init( &i_argc, ppsz_argv, ppsz_env );
 
 #elif defined( WIN32 )
@@ -432,7 +444,7 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * System specific cleaning code
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN ) || defined( WIN32 )
     system_End();
 #endif
 
@@ -692,6 +704,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_IMDCT:                                           /* --imdct */
             main_PutPszVariable( IMDCT_METHOD_VAR, optarg );
             break;
+        case OPT_VOLUME:                                         /* --volume */
+            main_PutIntVariable( AOUT_VOLUME_VAR, atoi(optarg) );
+            break;
 
         /* Video options */
         case OPT_NOVIDEO:                                       /* --novideo */
@@ -738,16 +753,16 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             break;
 
         /* DVD options */
-        case 't':
+        case 't':                                              /* --dvdtitle */
             main_PutIntVariable( INPUT_TITLE_VAR, atoi(optarg) );
             break;
-        case 'T':
+        case 'T':                                            /* --dvdchapter */
             main_PutIntVariable( INPUT_CHAPTER_VAR, atoi(optarg) );
             break;
-        case 'u':
+        case 'u':                                              /* --dvdangle */
             main_PutIntVariable( INPUT_ANGLE_VAR, atoi(optarg) );
             break;
-        case 'a':
+        case 'a':                                              /* --dvdaudio */
             if ( ! strcmp(optarg, "ac3") )
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_AC3 );
             else if ( ! strcmp(optarg, "lpcm") )
@@ -757,12 +772,18 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             else
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_NOAUDIO );
             break;
-        case 'c':
+        case 'c':                                            /* --dvdchannel */
             main_PutIntVariable( INPUT_CHANNEL_VAR, atoi(optarg) );
             break;
-        case 's':
+        case 's':                                           /* --dvdsubtitle */
             main_PutIntVariable( INPUT_SUBTITLE_VAR, atoi(optarg) );
             break;
+        case OPT_DVDCSS_METHOD:                           /* --dvdcss-method */
+            main_PutPszVariable( "DVDCSS_METHOD", optarg );
+            break;
+        case OPT_DVDCSS_VERBOSE:                         /* --dvdcss-verbose */
+            main_PutPszVariable( "DVDCSS_VERBOSE", optarg );
+            break;
 
         /* Input options */
         case OPT_INPUT:                                           /* --input */
@@ -771,6 +792,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_CHANNELS:                                     /* --channels */
             main_PutIntVariable( INPUT_NETWORK_CHANNEL_VAR, 1 );
             break;
+        case OPT_CHANNELSERVER:                           /* --channelserver */
+            main_PutPszVariable( INPUT_CHANNEL_SERVER_VAR, optarg );
+            break;
         case OPT_SERVER:                                         /* --server */
             main_PutPszVariable( INPUT_SERVER_VAR, optarg );
             break;
@@ -793,7 +817,7 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             ShowConsole();
             RedirectSTDOUT();
             intf_ErrMsg( "intf error: unknown option `%s'",
-                         ppsz_argv[optind - 1] );
+                         ppsz_argv[optind] );
             Usage( USAGE );
 #ifdef WIN32        /* Pause the console because it's destroyed when we exit */
             if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
@@ -866,6 +890,7 @@ static void Usage( int i_fashion )
           "\n      --spdif                    \tAC3 pass-through mode"
           "\n      --downmix <module>         \tAC3 downmix method"
           "\n      --imdct <module>           \tAC3 IMDCT method"
+          "\n      --volume [0..1024]         \tVLC output volume"
           "\n"
           "\n      --novideo                  \tdisable video"
           "\n  -V, --vout <module>            \tvideo output method"
@@ -888,9 +913,12 @@ static void Usage( int i_fashion )
           "\n  -a, --dvdaudio <type>          \tchoose DVD audio type"
           "\n  -c, --dvdchannel <channel>     \tchoose DVD audio channel"
           "\n  -s, --dvdsubtitle <channel>    \tchoose DVD subtitle channel"
+          "\n      --dvdcss-method <method>   \tselect dvdcss decryption method"
+          "\n      --dvdcss-verbose <level>   \tselect dvdcss verbose level"
           "\n"
           "\n      --input                    \tinput method"
           "\n      --channels                 \tenable channels"
+          "\n      --channelserver <host>     \tchannel server address"
           "\n      --server <host>            \tvideo server address"
           "\n      --port <port>              \tvideo server port"
           "\n      --broadcast                \tlisten to a broadcast"
@@ -917,6 +945,7 @@ static void Usage( int i_fashion )
         "\n  " AOUT_SPDIF_VAR "={1|0}                 \tAC3 pass-through mode"
         "\n  " DOWNMIX_METHOD_VAR "=<method name>     \tAC3 downmix method"
         "\n  " IMDCT_METHOD_VAR "=<method name>       \tAC3 IMDCT method"
+        "\n  " AOUT_VOLUME_VAR "=[0..1024]            \tVLC output volume"
         "\n  " AOUT_RATE_VAR "=<rate>             \toutput rate" );
 
     /* Video parameters */
@@ -938,22 +967,22 @@ static void Usage( int i_fashion )
 
     /* DVD parameters */
     intf_MsgImm( "\nDVD parameters:"
-        "\n  " INPUT_DVD_DEVICE_VAR "=<device>           \tDVD device"
-        "\n  " INPUT_TITLE_VAR "=<title>             \ttitle number"
-        "\n  " INPUT_CHAPTER_VAR "=<chapter>         \tchapter number"
-        "\n  " INPUT_ANGLE_VAR "=<angle>             \tangle number"
-        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off} \taudio type"
-        "\n  " INPUT_CHANNEL_VAR "=[0-15]            \taudio channel"
-        "\n  " INPUT_SUBTITLE_VAR "=[0-31]           \tsubtitle channel" );
+        "\n  " INPUT_DVD_DEVICE_VAR "=<device>        \tDVD device"
+        "\n  " INPUT_TITLE_VAR "=<title>              \ttitle number"
+        "\n  " INPUT_CHAPTER_VAR "=<chapter>          \tchapter number"
+        "\n  " INPUT_ANGLE_VAR "=<angle>              \tangle number"
+        "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off}  \taudio type"
+        "\n  " INPUT_CHANNEL_VAR "=[0-15]             \taudio channel"
+        "\n  " INPUT_SUBTITLE_VAR "=[0-31]            \tsubtitle channel" );
 
     /* Input parameters */
     intf_MsgImm( "\nInput parameters:"
-        "\n  " INPUT_SERVER_VAR "=<hostname>         \tvideo server"
-        "\n  " INPUT_PORT_VAR "=<port>               \tvideo server port"
-        "\n  " INPUT_IFACE_VAR "=<interface>         \tnetwork interface"
-        "\n  " INPUT_BCAST_ADDR_VAR "=<addr>         \tbroadcast mode"
-        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname> \tchannel server"
-        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>       \tchannel server port" );
+        "\n  " INPUT_SERVER_VAR "=<hostname>          \tvideo server"
+        "\n  " INPUT_PORT_VAR "=<port>            \tvideo server port"
+        "\n  " INPUT_IFACE_VAR "=<interface>          \tnetwork interface"
+        "\n  " INPUT_BCAST_ADDR_VAR "=<addr>          \tbroadcast mode"
+        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname>  \tchannel server"
+        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>        \tchannel server port" );
 
 }
 
@@ -1157,6 +1186,7 @@ static int CPUCapabilities( void )
     {
         i_capabilities |= CPU_CAPABILITY_MMXEXT;
 
+#ifdef HAVE_SSE
         /* We test if OS support the SSE instructions */
         i_illegal = 0;
         if( setjmp( env ) == 0 )
@@ -1178,6 +1208,7 @@ static int CPUCapabilities( void )
             fprintf( stderr, "(you will need Linux kernel 2.4.x or later)\n" );
 #endif
         }
+#endif
     }
     
     /* test for additional capabilities */
@@ -1192,6 +1223,7 @@ static int CPUCapabilities( void )
     /* list these additional capabilities */
     cpuid( 0x80000001 );
 
+#ifdef HAVE_3DNOW
     if( i_edx & 0x80000000 )
     {
         i_illegal = 0;
@@ -1206,6 +1238,7 @@ static int CPUCapabilities( void )
             i_capabilities |= CPU_CAPABILITY_3DNOW;
         }
     }
+#endif
 
     if( b_amd && ( i_edx & 0x00400000 ) )
     {
@@ -1216,9 +1249,11 @@ static int CPUCapabilities( void )
     return( i_capabilities );
 
 #elif defined( __powerpc__ )
+
     /* Test for Altivec */
     signal( SIGILL, InstructionSignalHandler );
 
+#ifdef HAVE_ALTIVEC
     i_illegal = 0;
     if( setjmp( env ) == 0 )
     {
@@ -1229,9 +1264,9 @@ static int CPUCapabilities( void )
     {
         i_capabilities |= CPU_CAPABILITY_ALTIVEC;
     }
+#endif
 
     signal( SIGILL, NULL );     
-
     return( i_capabilities );
 
 #else
